
# q-expansion of newform 525.2.a.f, downloaded from the LMFDB on 08 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 525
weight = 2
poly_data = [-3, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, -1], [-1, 0], [0, 0], [-1, 0], [-3, 0], [-2, 2], [0, -2], [2, 2], [-3, 4], [-3, -2], [2, -4], [-5, 4], [0, 0], [-5, -2], [-6, -2], [6, -4], [-6, -2], [8, -4], [-11, -2], [-3, 0], [4, -2], [-1, -6], [-6, 4], [-6, 6], [10, -4], [12, 2], [10, -6], [0, 0], [-7, 4], [-3, 6], [1, -6], [-12, 2], [-6, 4], [-4, 10], [-21, 2], [-7, 6], [4, -8], [-8, -4], [-6, 10], [-6, 6], [0, 0], [8, 2], [-12, 4], [1, -4], [-3, -6], [8, 0], [8, 8], [-20, 2], [-18, -2], [2, -4], [15, -6], [24, -4], [14, -2], [0, -6], [-6, 12], [15, -4], [6, -4], [-22, -2], [10, 0], [-15, 6], [-14, 12], [-6, -2], [4, 2], [0, 6], [10, 6], [21, -2], [11, -14], [-2, 4], [-21, -4], [-10, 18], [30, 0], [-3, 12], [4, -8], [1, 0], [-1, -6], [-6, 14], [-15, -6], [-2, 0], [-3, -6], [2, -6], [6, 0], [-1, -4], [24, -8], [28, 4], [2, 2], [12, -12], [-21, -6], [13, -4], [6, -14], [-8, -4], [-18, 8], [30, 2], [7, 6], [-3, 0], [-28, 0], [18, -4], [6, -2], [-18, -8], [16, 0], [-13, -8], [25, 2], [21, -14], [-6, -4], [-15, 6], [-1, -2], [-26, -2], [-24, -6], [0, -12], [15, -4], [8, 4], [10, 6], [31, 8], [-9, -6], [2, -2], [-1, 10], [9, 6], [-20, -8], [-6, 0], [-6, 4], [-12, 0], [-34, 6], [10, 0], [18, -4], [-9, 12], [32, 4], [-30, 4], [2, -16], [36, -4], [28, -2], [-20, 0], [17, -6], [0, 12], [-28, 12], [-41, 0], [0, -12], [-4, -12], [-6, 0], [4, -14], [-12, 18], [-9, 18], [-22, 16], [-6, 24], [-11, -6], [3, 12], [32, -14], [-36, 6], [16, -4], [0, -12], [-22, -12], [-33, 12], [22, 0], [6, 12], [-11, -2], [24, -4], [4, 4], [15, 0], [-13, -2], [12, -14], [-8, 28], [6, 0], [-36, 16], [9, 2], [4, 16], [-6, 26], [-21, 10], [6, -20], [23, 10], [-26, 18]]
